==> CGAL-5.6.3.tar.xz: CGAL-5.6.3/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h <==
// Copyright (c) 2014
// INRIA Saclay-Ile de France (France)
//
// This file is part of CGAL (www.cgal.org)
//
// $URL: https://github.com/CGAL/cgal/blob/v5.6.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h $
// $Id: determinant_of_points_from_vectors.h 07793738355 2020-03-26T13:31:46+01:00 Sébastien Loriot
// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
//
// Author(s)     : Marc Glisse

#ifndef CGAL_VECTOR_DETPTS_H
#define CGAL_VECTOR_DETPTS_H
#include <CGAL/NewKernel_d/functor_tags.h>
#include <CGAL/Dimension.h>

namespace CGAL {

template <class LA, class Dim_=typename LA::Dimension,
         class Max_dim_=typename LA::Max_dimension,
         bool = LA::template Property<Has_determinant_of_points_tag>::value,
         bool = LA::template Property<Has_determinant_of_vectors_tag>::value
           && LA::template Property<Has_vector_plus_minus_tag>::value>
struct Add_determinant_of_points_from_vectors_and_minus : LA {
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
};

//FIXME: Use variadics and boost so it works in any dimension.
template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<2>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    boost::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c){
    return LA::determinant_of_vectors(b-a,c-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c){
    return LA::sign_of_determinant_of_vectors(b-a,c-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<3>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    boost::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d){
    return LA::determinant_of_vectors(b-a,c-a,d-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<4>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    boost::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<5>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    boost::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a,f-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a,f-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<6>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    boost::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f,
      Vector const&g){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a,f-a,g-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f,
      Vector const&g){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a,f-a,g-a);
  }
};

}
#endif

==> CGAL-6.0.3.tar.xz: CGAL-6.0.3/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h <==
// Copyright (c) 2014
// INRIA Saclay-Ile de France (France)
//
// This file is part of CGAL (www.cgal.org)
//
// $URL: https://github.com/CGAL/cgal/blob/v6.0.3/NewKernel_d/include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h $
// $Id: include/CGAL/NewKernel_d/Vector/determinant_of_points_from_vectors.h cefe3007d59 $
// SPDX-License-Identifier: LGPL-3.0-or-later OR LicenseRef-Commercial
//
// Author(s)     : Marc Glisse

#ifndef CGAL_VECTOR_DETPTS_H
#define CGAL_VECTOR_DETPTS_H
#include <CGAL/NewKernel_d/functor_tags.h>
#include <CGAL/Dimension.h>

namespace CGAL {

template <class LA, class Dim_=typename LA::Dimension,
         class Max_dim_=typename LA::Max_dimension,
         bool = LA::template Property<Has_determinant_of_points_tag>::value,
         bool = LA::template Property<Has_determinant_of_vectors_tag>::value
           && LA::template Property<Has_vector_plus_minus_tag>::value>
struct Add_determinant_of_points_from_vectors_and_minus : LA {
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
};

//FIXME: Use variadics and boost so it works in any dimension.
template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<2>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    std::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c){
    return LA::determinant_of_vectors(b-a,c-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c){
    return LA::sign_of_determinant_of_vectors(b-a,c-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<3>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    std::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d){
    return LA::determinant_of_vectors(b-a,c-a,d-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<4>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    std::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<5>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    std::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a,f-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a,f-a);
  }
};

template <class LA, class Max_dim_>
struct Add_determinant_of_points_from_vectors_and_minus
<LA, Dimension_tag<6>, Max_dim_, false, true> : LA {
  typedef typename LA::NT NT;
  typedef typename LA::Vector Vector;
  template< class D2, class D3=D2 >
    struct Rebind_dimension {
      typedef typename LA::template Rebind_dimension<D2,D3> LA2;
      typedef Add_determinant_of_points_from_vectors_and_minus<LA2> Other;
    };
  template<class P,class=void> struct Property : LA::template Property<P> {};
  template<class D> struct Property<Has_determinant_of_points_tag, D> :
    std::true_type {};

  static NT determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f,
      Vector const&g){
    return LA::determinant_of_vectors(b-a,c-a,d-a,e-a,f-a,g-a);
  }
  static Sign sign_of_determinant_of_points(Vector const&a, Vector const&b,
      Vector const&c, Vector const&d, Vector const&e, Vector const&f,
      Vector const&g){
    return LA::sign_of_determinant_of_vectors(b-a,c-a,d-a,e-a,f-a,g-a);
  }
};

}
#endif
